/**
  * Created by ariellemoro on 29.11.16.
  */

import collection.mutable.{ListBuffer, Map}

object MarkovChainAlgo {

  def main(args: Array[String]): Unit = {
    val stateMap = Map(0 -> "Copeaux", 1 -> "Roue", 2 -> "Mangeoire")
    val stateSequence = List(1,2,0,1,1,1,2,0,1,2,1,2,0,1,2,1,0,0,0,1,2,1,2,1,0,2,0,2,1,2,1,2,1,1,1,0,1,0,1,0,1,0)
    //Create the Markov Chain of "Doudou le hamster"
    val markovChain = createMarkovChain(stateMap, stateSequence)
    //Print the Markov Chain
    val stateNB = stateMap.size
    var indexState = 0
    while(indexState < stateNB){
      val nextStates = markovChain(indexState)
      val state = stateMap(indexState)
      println(s"From $state")
      var indexNextState = 0
      while(indexNextState < stateNB){
        val nextState = stateMap(indexNextState)
        val prob = nextStates(indexNextState)
        println(s"   -> to $nextState: $prob %")
        indexNextState = indexNextState + 1
      }
      indexState = indexState + 1
    }
    //Where Doudou will go next?
    val currentStateIndex = 0 //"Copeaux"
    val currentLine = markovChain(currentStateIndex)
    var bestProb = 0.0
    var bestStateIndex = 0
    var ind = 0
    for(p <- currentLine){
      if(p > bestProb){
        bestProb = p
        bestStateIndex = ind
      }
      ind = ind + 1
    }
    val fromState = stateMap(currentStateIndex)
    val toState = stateMap(bestStateIndex)
    println(s"Doudou will likely go to $toState from $fromState")
  }

  //Function helping to create a Markov Chain
  def createMarkovChain(state:Map[Int, String], sequence: List[Int]): ListBuffer[ListBuffer[Double]] = {
    //Create an empty Markov Chain with 0 for each transition between 2 states
    var tempMarkovChain = new ListBuffer[ListBuffer[Double]] //Structure => [[2,3,5], [0,1,2], [9,2,6]] (ex) with transition number only, not probabilities (index are used to identify the number of states)
    val stateNB = state.size
    var indexState = 0
    while(indexState < stateNB){
      val nextStates = new ListBuffer[Double]
      var indexNextStates = 0
      while(indexNextStates < stateNB){
        nextStates += 0
        indexNextStates = indexNextStates + 1
      }
      tempMarkovChain += nextStates
      indexState = indexState + 1
    }
    //Fill the temp Markov Chain according to the state sequence
    val sequenceSize = sequence.length
    var index = 0
    while(index < sequenceSize - 1){
      //Extract transition
      val firstState = sequence(index)
      val secondState = sequence(index+1)
      //Update the next state line
      val nextStateLine = tempMarkovChain(firstState)
      val newValue = nextStateLine(secondState) + 1
      nextStateLine(secondState) = newValue
      index = index + 1
    }
    //Convert the matrix transition number into transition probabilities (the sum of all departure transition probabilities of a state must equal to 0)
    var indexState2 = 0
    while(indexState2 < stateNB){
      val nextStateLine = tempMarkovChain(indexState2)
      //Compute the sum of the current line
      var sumLine = 0.0
      var indexNextStates2 = 0
      while(indexNextStates2 < stateNB){
        sumLine = sumLine + nextStateLine(indexNextStates2)
        indexNextStates2 = indexNextStates2 + 1
      }
      //Compute the transition probability of each next state
      indexNextStates2 = 0
      while(indexNextStates2 < stateNB){
        val currentProba = nextStateLine(indexNextStates2) / sumLine
        nextStateLine(indexNextStates2) = currentProba
        indexNextStates2 = indexNextStates2 + 1
      }
      indexState2 = indexState2 + 1
    }
    return tempMarkovChain
  }

}
